Monomero[] m; //vetor de monômeros 

Botao[] especie; //vetor de botões que definem o polimero a ser apresentado

Botao[] processo; //vetor de botões utilizados na dinâmica da animação

Credito credito;

PFont f; //fonte

float comprimento; //comprimento do monômero de referência (etileno) --> equivale ao dobro do comprimento da ligação dupla

float vx; //velocidade com que os radicais e os monômeros se movimentam

//float omega; //velocidade angular de abertura da ligação (NÃO UTILIZADO!)

String etapa; //guarda o nome da etapa da animação que está sendo apresentada,
              //pode guardar "MONÔMERO", "INICIAÇÃO", "PROPAGAÇÃO" ou "TERMINAÇÃO"

String polimero; //guarda o nome do polimero que esta sendo apresentado

//VARIÁVEIS DA ANIMAÇÃO
float rx; //posição no eixo X do iniciador
float rx_; //posição no eixo X do terminador

float lado = 1; //binário utilizado para gerenciar a alternância dos sentidos de monômeros sucessivos

//SETUP////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup(){
  size(700, 700);
  smooth();
  strokeWeight(3);
  textAlign(CENTER);
  
  //DEFINIÇÃO DAS CONSTANTES
  comprimento = 60;
  vx = 3;
 // omega = PI/100;
  
  //INICIALIZAÇÕES
  f = loadFont("CourierNewPS-BoldMT-28.vlw"); 
  textFont(f, 38); //escrevemos o título com uma letra maior
  
  etapa = "MONÔMERO"; //primeiramente apresentamos o monômero isolado
  
  rx = - comprimento; //o radical surge de fora da tela, entrando pelo lado esquerdo
  
  polimero = "POLIETILENO"; //por ser o monômero mais simples, o polietileno foi escolhido
                            //para ser o polímero cujo processo é apresentado inicialmente
  
  //iniciamos a matriz de monômeros
  m = new Monomero[1]; 
  m[0] = new Monomero(width/2, height/2 , 0); //colocamos o primeiro monômero no centro da tela
  
  //inicialização dos botões dos polímeros
  especie = new Botao[5];
  especie[0] = new Botao(width/2 - 2*120, height/4, "POLIETILENO");         //polietileno
    //o botão referente ao polietileno esta inicialmente selecionado
    especie[0].situacao = "SELECIONADO";
    especie[0].psituacao = "LIBERADO";
  especie[1] = new Botao(width/2 - 1*120, height/4, "TEFLON");              //teflon
  especie[2] = new Botao(width/2 - 0*120, height/4, "PVC");                 //PVC
  especie[3] = new Botao(width/2 + 1*120, height/4, "POLIPROPILENO");       //polipropileno
  especie[4] = new Botao(width/2 + 2*120, height/4, "ACRÍLICO");            //acrílico
  
  //inicialização dos botões de animação
  processo = new Botao[2];
  processo[0] = new Botao(width/2 + 1*130, 7*height/10, "INÍCIO");           //início
    /*processo[0].situacao = "LIBERADO";
    processo[0].psituacao = "SELECIONADO";*/
  processo[1] = new Botao(width/2 + 2*130, 7*height/10, "AVANÇAR >>");       //avançar
  
  //inicialização do botão de créditos
  credito = new Credito();
  
  //inicializamos a tela (definindo o fundo branco) e escrevemos os textos uma única vez
  //atualizando apenas a área em que a animação ocorre por meio do desenho de um retângulo
  //que cobre tal região  
  
  background(255); // fundo branco
  
  //TEXTO
  fill(0);
  text("POLIMERIZAÇÃO:\nCadeia", width/2, height/12); //título
  
  textFont(f, 12); //diminuimos a fonta para a apresentação dos botões de polímeros
  text("Polímero formado:", width/10, height/5);
  
   //apresentamos o texto explicativo
  textFont(f, 14);
  text("O monômero inicialmente estável passa a apresentar um sítio reativo\n" +
       "a partir do momento em que sua dupla ligação se quebra na presença\n" +
       "de um iniciador (R).\n\n" +
       "O sítio reativo do mero estimula a quebra da ligação dupla de uma\n" +
       "próxima molécula de monômero, que se adiciona à corrente\n" +
       "polimérica até que um terminador (R') se adicione ao polímer0\n" +
       " e cancele a reatividade do último mero da sequência.", width/2, 4*height/5);   
}

//DRAW/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void draw(){
  fill(255);  //colorimos o retângulo
  noStroke();
  rectMode(CORNERS);
  rect(0, m[0].ypos - height/6, width, m[0].ypos + height/6); //atualizamos a área de animação  
  anima(); 
  botaoDisplay();
  polimeroDisplay();
  legenda(width/15, 3*height/5);
  credito.display();
}

/////////FUNÇÕES DO PROGRAMA///////////////////////////////////////////////////////////////////////////////////////////////////////////

//desenvolve a animação
void anima(){
  
  float r0 = dist(rx, m[0].ypos, m[0].xpos, m[0].ypos); //distância entre o radical de iniciação ou de terminação 
                                                        //e o centro da ligação dupla principal do primeiro monômero
  
  //a função age, obviamente, de forma diferente em cada etapa da animação
  
  //iniciação
  if(etapa == "INICIAÇÃO")
    {
      textFont(f, 28);
      
      //o radical se aproxima do primeiro monômero até assumir a distância de ligação
      if(r0 > 5*comprimento/6)
        rx += vx;
        
      //com a aproximação do radical, abrimos a ligação do monômero, após ultrapassada
      //uma distância mínima  
      if(r0 < 10*comprimento/6)
        m[0].abrir(vx*PI*6/(5*comprimento));
    }
   
   //desenhamos o radical R (com exceção da etapa MONÔMERO)
   if(etapa != "MONÔMERO"){
      textAlign(CENTER);     
      pushMatrix();
      translate(rx, m[0].ypos);
      stroke(0);
      line(0, 0, comprimento/3, 0);
      fill(#FFA255); //LARANJA
      noStroke();
      ellipse(0, 0, comprimento/2, comprimento/2); //radical
      fill(0);
      text("R", 0, comprimento/10); //nomeamos o radical
      popMatrix();
   }
  
  //propagação 
  if(etapa == "PROPAGAÇÃO"){
    textFont(f, 18);
    
    //propagamos a polimerização por meio da criação de novos objetos da classe
    //Monomero, que se movimentam em direção do último objeto desta classe até
    //que seja atingida a distância de ligação, quando o objeto cessa seu movimento
    //e é, então, criado um novo Monomero que executa o mesmo processo.
    
    if(m.length < 7){ //criamos um numero máximo de 7 monomeros
    
      //a distância de ligação é a condição para se criar um novo objeto   
      if(m[m.length - 1].xpos - m[0].xpos <= (m.length -1)*comprimento){ 
        //criamos e iniciamos o novo Monomero
        Monomero novo = new Monomero(width + 3*comprimento/2, m[0].ypos, 0);
        m = (Monomero[]) append(m, novo);
      }
      //se a distância ainda não for atingida, movimentamos o último monômero
      else
        {
         m[m.length - 1].andar(-vx, 0);
         //do mesmo modo que ocorre na iniciação, abrimos a ligação do último monômero após
         //atingida uma distância mínima
         if(m[m.length - 1].xpos - m[0].xpos < comprimento*(m.length))
           m[m.length - 1].abrir(vx*PI/comprimento);
        }
      }
  }
  
  //terminação    
  if(etapa == "TERMINAÇÃO"){
    
    //nesta etapa executa-se um processo semelhante ao que aparece na iniciação,
    //porém criamos um radical de terminação que se aproxima pela direita, até
    //atingir a distância de ligação
    
    if(m.length == 7)
      r0 = dist(rx_, m[m.length - 2].ypos, m[m.length - 2].xpos, m[m.length - 2].ypos);
    else
      r0 = dist(rx_, m[m.length - 1].ypos, m[m.length - 1].xpos, m[m.length - 1].ypos);          
    if(r0 > 5*comprimento/6){
        rx_ -= vx;
    }
    pushMatrix();
    translate(rx_, m[m.length - 2].ypos);
    
    //desenhamos o radical terminador R'
    stroke(0);
    line(0, 0, -comprimento/3, 0);
    fill(#FC59E2);//ROSA
    noStroke();
    ellipse(0, 0, comprimento/2, comprimento/2);
    fill(0);
    text("R'", comprimento/30, comprimento/10);
    popMatrix();
    }  
  }  

//função que desenha todos os botões      
void botaoDisplay(){
  rectMode(CENTER); 
  textFont(f, 12);
  for(int i = 0; i < especie.length; i++)
    especie[i].display();
  for(int i = 0; i < processo.length; i++)
    processo[i].display();
}  

//função que desenha os polímeros, acionando o método relativo ao polímero escolhido
//em cada um dos objetos Monomero
void polimeroDisplay(){
  if(polimero == "POLIETILENO")
    for(int i = 0; i < m.length; i++)
      m[i].displayEthylene();
  if(polimero == "TEFLON")
    for(int i = 0; i < m.length; i++)
      m[i].displayTFEthylene();
  if(polimero == "PVC")
    for(int i = 0; i < m.length; i++)
      m[i].displayPVC();
  if(polimero == "POLIPROPILENO")
    for(int i = 0; i < m.length; i++)
      m[i].displayPropylene();
  if(polimero == "ACRÍLICO")
    for(int i = 0; i < m.length; i++)
      m[i].displayAcrylic();   
}

void legenda(float x, float y){
  textAlign(LEFT);
  textFont(f, 12);
  pushMatrix();
  translate(x, y);
  fill(255);
  rect(70, 40, 140, 100);
  fill(0);
  noStroke();
  text("Legenda:", 0, 0);
  text("Carbono", 60, 15);
  fill(#91D6F7); //AZUL
  ellipse(50, 11, 10, 10);
  fill(0);
  text("Hidrogênio", 60, 30);
  fill(122);
  ellipse(50, 26, 10, 10);
  fill(0);
  text("Flúor", 60, 45);
  fill(#BC90FA);
  ellipse(50, 41, 10, 10);
  fill(0);
  text("Oxigênio", 60, 60);
  fill(#FC928F);
  ellipse(50, 56, 10, 10);
  popMatrix();
  textAlign(CENTER);
}
/////////MOUSE///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////  

//ao clique do mouse:
void mousePressed(){
  
  //analisamos se os botões foram acionados
   for(int i = 0; i < especie.length; i++)
     especie[i].mousePressed();
   
   for(int i = 0; i < processo.length; i++)
     processo[i].mousePressed(); 
  
  //para que não sobrem resquícios dos créditos nas áreas não atualizadas, as reescrevemos quando
  //retiramos os creditos
  
  if(credito.botao.situacao == "SELECIONADO" ){
    
    background(255); // fundo branco
  
    //TEXTO
    fill(0);
    
    textFont(f, 38); //escrevemos o título com uma letra maior
    text("POLIMERIZAÇÃO:\nCadeia", width/2, height/12); //título
  
    textFont(f, 12); //diminuimos a fonta para a apresentação dos botões de polímeros
    text("Polímero formado:", width/10, height/5);
  
     //apresentamos o texto explicativo
    textFont(f, 14);
    text("O monômero inicialmente estável passa a apresentar um sítio reativo\n" +
         "a partir do momento em que sua dupla ligação se quebra na presença\n" +
         "de um iniciador (R).\n\n" +
         "O sítio reativo do mero estimula a quebra da ligação dupla de uma\n" +
         "próxima molécula de monômero, que se adiciona à corrente\n" +
         "polimérica até que um terminador (R') se adicione ao polímer0\n" +
         " e cancele a reatividade do último mero da sequência.", width/2, 4*height/5);    
   }
   
   fill(255);  //colorimos o retângulo
    noStroke();
    rectMode(CORNERS);
    rect(0, m[0].ypos - height/6, width, m[0].ypos + height/6); //atualizamos a área de animação  
    anima(); 
    botaoDisplay();
    polimeroDisplay();
    legenda(width/15, 3*height/5); 
   
   //atualizamos a situação do botão de créditos
   credito.botao.mousePressed();  
   credito.display(); 
   
   /*if(mouseButton == LEFT)
     etapa ++;
   
   if(mouseButton == RIGHT || etapa > 3){
     etapa = 0;
     comprimento = 60;
     rx = - comprimento;
     m = new Monomero[1];
     m[0] = new Monomero(width/2, height/2 - 40, 0);
   }*/
     
   
  
/*  play = !play;
  Monomero novo;
  novo = new Monomero(width/2 - m.length*(comprimento), height/2, 0);
  if(!play)
    m = (Monomero[]) append(m, novo);
 */
}

//ao soltar o botão do mouse  
void mouseReleased(){
  
  //se algum dos botões de polímeros foi acionado com o clique,
  //os outros são liberados
  for(int i = 0; i < especie.length; i++){
    if(especie[i].situacao == "PRESSIONADO"){
      for(int j = 0; j < especie.length; j++)
        if(j != i)
          especie[j].libera();
      polimero = especie[i].texto;
    }
      especie[i].mouseReleased(); //se o botão foi acionado pelo clique, gerenciamos sua situação    
  }
    
  //se o botão acionado foi 'início', inicializamos a animação  
  if(processo[0].situacao == "PRESSIONADO"){
    etapa = "MONÔMERO";
    lado = 1;
    comprimento = 60;
    rx = - comprimento;
    vx = 3;
    m = new Monomero[1];
    m[0] = new Monomero(width/2, height/2 , 0);
  }
  
  //se o botão de Crédito for acionado, apresentamos ou deixamos de apresentar o texto
  
  credito.botao.mouseReleased();
  
  //se o botão acionado foi 'avançar', modificamos o valor de 'etapa' para o próximo processo  
  if(processo[1].situacao == "PRESSIONADO"){
    //'MONOMERO' --> 'INICIAÇÃO'
    if(etapa == "MONÔMERO")
      etapa = "INICIAÇÃO";
    //'INICIAÇÃO' --> 'PROPAGAÇÃO'  
    else if(etapa == "INICIAÇÃO"){
      comprimento = 40; //a etapa propagação adota uma escala reduzida, facilitando a visualização do processo
      m[0].xpos = width/2 - 3*comprimento;
      m[0].angle = PI;
      rx = m[0].xpos - 5*comprimento/6;
      vx = 5; //para agilizar a etapa de propagação, aumentamos a velocidade
      etapa = "PROPAGAÇÃO";      
      textFont(f, 18);
    }
    //PROPAGAÇÃO --> TERMINAÇÃO
    else if(etapa == "PROPAGAÇÃO"){
      etapa = "TERMINAÇÃO";
      //colocamos o ultimo objeto Monomero em sua posição final para garantir este
      //posicionamento
      if(m.length < 7){
        m[m.length - 1].xpos = m[0].xpos + (m.length -1)*comprimento;
        m[m.length - 1].angle = PI;
      }    
      rx_ = width + 3*comprimento/2;//o radical de terminação vem de fora da tela, aproximando-se pela direita
      vx = 3;
    }
  }
  
  
 /* for(int i = 0; i < processo.length; i++){
    if(processo[i].situacao == "PRESSIONADO")
      for(int j = 0; j <processo.length; j++)
          processo[j].libera();
    } */ 
    
    //os botões de animação estão sempre liberados, atuando apenas quando pressionados
    processo[0].libera();
    processo[1].libera();   
}
    


